import 'package:equatable/equatable.dart';
import 'package:flutter/material.dart';
import 'package:hydrated_bloc/hydrated_bloc.dart';

import 'package:hangman/config/default_game_settings.dart';
import 'package:hangman/models/settings/settings_game.dart';

part 'settings_game_state.dart';

class GameSettingsCubit extends HydratedCubit<GameSettingsState> {
  GameSettingsCubit() : super(GameSettingsState(settings: GameSettings.createDefault()));

  void setValues({
    String? gameMode,
    String? gameLevel,
  }) {
    emit(
      GameSettingsState(
        settings: GameSettings(
          gameMode: gameMode ?? state.settings.gameMode,
          gameLevel: gameLevel ?? state.settings.gameLevel,
        ),
      ),
    );
  }

  String getParameterValue(String code) {
    switch (code) {
      case DefaultGameSettings.parameterCodeGameMode:
        return GameSettings.getGameModeValueFromUnsafe(state.settings.gameMode);
      case DefaultGameSettings.parameterCodeGameLevel:
        return GameSettings.getGameLevelValueFromUnsafe(state.settings.gameLevel);
    }

    return '';
  }

  void setParameterValue(String code, String value) {
    final String gameMode = code == DefaultGameSettings.parameterCodeGameMode
        ? value
        : getParameterValue(DefaultGameSettings.parameterCodeGameMode);
    final String gameLevel = code == DefaultGameSettings.parameterCodeGameLevel
        ? value
        : getParameterValue(DefaultGameSettings.parameterCodeGameLevel);

    setValues(
      gameMode: gameMode,
      gameLevel: gameLevel,
    );
  }

  @override
  GameSettingsState? fromJson(Map<String, dynamic> json) {
    final String gameMode = json[DefaultGameSettings.parameterCodeGameMode] as String;
    final String gameLevel = json[DefaultGameSettings.parameterCodeGameLevel] as String;

    return GameSettingsState(
      settings: GameSettings(
        gameMode: gameMode,
        gameLevel: gameLevel,
      ),
    );
  }

  @override
  Map<String, dynamic>? toJson(GameSettingsState state) {
    return <String, dynamic>{
      DefaultGameSettings.parameterCodeGameMode: state.settings.gameMode,
      DefaultGameSettings.parameterCodeGameLevel: state.settings.gameLevel,
    };
  }
}
